@@ -23,8 +23,11 @@
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <math.h>
+#include <stdlib.h>
 
 #include "gpm-graph-widget.h"
+#include "gpm-info.h"
+#include "gpm-info-data.h"
 #include "gpm-debug.h"
 
 G_DEFINE_TYPE (GpmGraph, gpm_graph, GTK_TYPE_DRAWING_AREA);
@@ -32,94 +35,151 @@ G_DEFINE_TYPE (GpmGraph, gpm_graph, GTK_TYPE_DRAWING_AREA);
 
 struct GpmGraphPrivate
 {
-	gboolean	use_grid;
-	gboolean	use_legend;
+	gboolean		 use_grid;
+	gboolean		 use_legend;
+	gboolean		 invert_x;
+	gboolean		 autorange_x;
+	gboolean		 invert_y;
 
-	gboolean	invert_x;
-	gboolean	invert_y;
+	gint			 stop_x;
+	gint			 stop_y;
+	gint			 start_x;
+	gint			 start_y;
+	gint			 box_x; /* size of the white box, not the widget */
+	gint			 box_y;
+	gint			 box_width;
+	gint			 box_height;
 
-	gint		stop_x;
-	gint		stop_y;
-
-	gint		box_x; /* size of the white box, not the widget */
-	gint		box_y;
-	gint		box_width;
-	gint		box_height;
-
-	float		unit_x; /* 10th width of graph */
-	float		unit_y; /* 10th width of graph */
+	float			 unit_x; /* 10th width of graph */
+	float			 unit_y; /* 10th width of graph */
 
 	GpmGraphAxisType	 axis_x;
 	GpmGraphAxisType	 axis_y;
 	cairo_font_options_t	*options;
 
-	GList		*list;
+	GList			*list;
+	GList			*events;
 };
 
 static gboolean gpm_graph_expose (GtkWidget *graph, GdkEventExpose *event);
 static void	gpm_graph_finalize (GObject *object);
 
-
 /**
  * gpm_graph_event_description:
- * @event: The event type, e.g. GPM_GRAPH_EVENT_SCREEN_DIM
- * @colour: The colour enum, e.g. GPM_GRAPH_COLOUR_DARK_BLUE (returned)
+ * @event: The event type, e.g. GPM_GRAPH_EVENT_SUSPEND
+ * Return value: a string value describing the event
  **/
 const char *
-gpm_graph_event_description (GpmGraphEvent    event,
-			     GpmGraphColour  *colour)
+gpm_graph_event_description (GpmGraphEvent event)
 {
-	const char *event_desc;
-	if (event == GPM_GRAPH_EVENT_AC_REMOVED) {
-		event_desc = _("AC power");
-		*colour = GPM_GRAPH_COLOUR_BLUE;
-	} else if (event == GPM_GRAPH_EVENT_LOW_POWER) {
-		event_desc = _("Low Power");
-		*colour = GPM_GRAPH_COLOUR_DARK_BLUE;
+	if (event == GPM_GRAPH_EVENT_ON_AC) {
+		return _("On AC");
+	} else if (event == GPM_GRAPH_EVENT_ON_BATTERY) {
+		return _("On battery");
 	} else if (event == GPM_GRAPH_EVENT_SCREEN_DIM) {
-		event_desc = _("Screen dim");
-		*colour = GPM_GRAPH_COLOUR_YELLOW;
+		return _("LCD dim");
+	} else if (event == GPM_GRAPH_EVENT_SCREEN_RESUME) {
+		return _("LCD resume");
 	} else if (event == GPM_GRAPH_EVENT_DPMS_OFF) {
-		event_desc = _("Screen off");
-		*colour = GPM_GRAPH_COLOUR_DARK_YELLOW;
+		return _("DPMS off");
+	} else if (event == GPM_GRAPH_EVENT_DPMS_ON) {
+		return _("DPMS on");
 	} else if (event == GPM_GRAPH_EVENT_SUSPEND) {
-		event_desc = _("Suspend");
-		*colour = GPM_GRAPH_COLOUR_RED;
+		return _("Suspend");
+	} else if (event == GPM_GRAPH_EVENT_RESUME) {
+		return _("Resume");
 	} else if (event == GPM_GRAPH_EVENT_HIBERNATE) {
-		event_desc = _("Hibernate");
-		*colour = GPM_GRAPH_COLOUR_DARK_RED;
+		return _("Hibernate");
+	} else if (event == GPM_GRAPH_EVENT_LID_CLOSED) {
+		return _("Lid closed");
+	} else if (event == GPM_GRAPH_EVENT_LID_OPENED) {
+		return _("Lid opened");
+	} else if (event == GPM_GRAPH_EVENT_NOTIFICATION) {
+		return _("Notification");
 	} else {
-		event_desc = NULL;
-		*colour = 0;
+		return _("Unknown event!");
+	}
+}
+
+/**
+ * gpm_graph_event_colour:
+ * @event: The event type, e.g. GPM_GRAPH_EVENT_SUSPEND
+ * Return value: a colout, e.g. GPM_GRAPH_COLOUR_DARK_BLUE
+ **/
+GpmGraphColour
+gpm_graph_event_colour (GpmGraphEvent event)
+{
+	if (event == GPM_GRAPH_EVENT_ON_AC) {
+		return GPM_GRAPH_COLOUR_BLUE;
+	} else if (event == GPM_GRAPH_EVENT_ON_BATTERY) {
+		return GPM_GRAPH_COLOUR_DARK_BLUE;
+	} else if (event == GPM_GRAPH_EVENT_SCREEN_DIM) {
+		return GPM_GRAPH_COLOUR_CYAN;
+	} else if (event == GPM_GRAPH_EVENT_SCREEN_RESUME) {
+		return GPM_GRAPH_COLOUR_DARK_CYAN;
+	} else if (event == GPM_GRAPH_EVENT_DPMS_OFF) {
+		return GPM_GRAPH_COLOUR_DARK_YELLOW;
+	} else if (event == GPM_GRAPH_EVENT_DPMS_ON) {
+		return GPM_GRAPH_COLOUR_YELLOW;
+	} else if (event == GPM_GRAPH_EVENT_SUSPEND) {
+		return GPM_GRAPH_COLOUR_RED;
+	} else if (event == GPM_GRAPH_EVENT_RESUME) {
+		return GPM_GRAPH_COLOUR_DARK_RED;
+	} else if (event == GPM_GRAPH_EVENT_HIBERNATE) {
+		return GPM_GRAPH_COLOUR_MAGENTA;
+	} else if (event == GPM_GRAPH_EVENT_LID_CLOSED) {
+		return GPM_GRAPH_COLOUR_GREEN;
+	} else if (event == GPM_GRAPH_EVENT_LID_OPENED) {
+		return GPM_GRAPH_COLOUR_DARK_GREEN;
+	} else if (event == GPM_GRAPH_EVENT_NOTIFICATION) {
+		return GPM_GRAPH_COLOUR_GREY;
+	} else {
+		return GPM_GRAPH_COLOUR_DEFAULT;
 	}
-	return event_desc;
 }
 
 /**
  * gpm_graph_set_axis_x:
- * @graph: This simple graph class instance
+ * @graph: This graph class instance
  * @axis: The axis type, e.g. GPM_GRAPH_TYPE_TIME
  **/
 void
 gpm_graph_set_axis_x (GpmGraph *graph, GpmGraphAxisType axis)
 {
+	g_return_if_fail (graph != NULL);
+	g_return_if_fail (GPM_IS_GRAPH (graph));
 	graph->priv->axis_x = axis;
 }
 
 /**
  * gpm_graph_set_axis_y:
- * @graph: This simple graph class instance
+ * @graph: This graph class instance
  * @axis: The axis type, e.g. GPM_GRAPH_TYPE_TIME
  **/
 void
 gpm_graph_set_axis_y (GpmGraph *graph, GpmGraphAxisType axis)
 {
+	g_return_if_fail (graph != NULL);
+	g_return_if_fail (GPM_IS_GRAPH (graph));
 	graph->priv->axis_y = axis;
 }
 
+/**
+ * gpm_graph_enable_legend:
+ * @graph: This graph class instance
+ * @enable: If we should show the legend
+ **/
+void
+gpm_graph_enable_legend	(GpmGraph *graph, gboolean enable)
+{
+	g_return_if_fail (graph != NULL);
+	g_return_if_fail (GPM_IS_GRAPH (graph));
+	graph->priv->use_legend = enable;
+}
+
 /**
  * gpm_graph_class_init:
- * @class: This simple graph class instance
+ * @class: This graph class instance
  **/
 static void
 gpm_graph_class_init (GpmGraphClass *class)
@@ -135,7 +195,7 @@ gpm_graph_class_init (GpmGraphClass *class)
 
 /**
  * gpm_graph_init:
- * @graph: This simple graph class instance
+ * @graph: This graph class instance
  **/
 static void
 gpm_graph_init (GpmGraph *graph)
@@ -143,10 +203,13 @@ gpm_graph_init (GpmGraph *graph)
 	graph->priv = GPM_GRAPH_GET_PRIVATE (graph);
 	graph->priv->invert_x = FALSE;
 	graph->priv->invert_y = FALSE;
+	graph->priv->start_x = 0;
+	graph->priv->start_y = 0;
 	graph->priv->stop_x = 60;
 	graph->priv->stop_y = 100;
 	graph->priv->use_grid = TRUE;
-	graph->priv->use_legend = TRUE;
+	graph->priv->use_legend = FALSE;
+	graph->priv->autorange_x = TRUE;
 	graph->priv->list = NULL;
 	graph->priv->axis_x = GPM_GRAPH_TYPE_TIME;
 	graph->priv->axis_y = GPM_GRAPH_TYPE_PERCENTAGE;
@@ -156,7 +219,7 @@ gpm_graph_init (GpmGraph *graph)
 
 /**
  * gpm_graph_finalize:
- * @object: This simple graph class instance
+ * @object: This graph class instance
  **/
 static void
 gpm_graph_finalize (GObject *object)
@@ -167,7 +230,7 @@ gpm_graph_finalize (GObject *object)
 
 /**
  * gpm_graph_set_invert_x:
- * @graph: This simple graph class instance
+ * @graph: This graph class instance
  * @inv: If we should invert the axis
  *
  * Sets the inverse policy for the X axis, i.e. to count from 0..X or X..0
@@ -175,12 +238,14 @@ gpm_graph_finalize (GObject *object)
 void
 gpm_graph_set_invert_x (GpmGraph *graph, gboolean inv)
 {
+	g_return_if_fail (graph != NULL);
+	g_return_if_fail (GPM_IS_GRAPH (graph));
 	graph->priv->invert_x = inv;
 }
 
 /**
  * gpm_graph_set_invert_y:
- * @graph: This simple graph class instance
+ * @graph: This graph class instance
  * @inv: If we should invert the axis
  *
  * Sets the inverse policy for the Y axis, i.e. to count from 0..Y or Y..0
@@ -188,12 +253,14 @@ gpm_graph_set_invert_x (GpmGraph *graph, gboolean inv)
 void
 gpm_graph_set_invert_y (GpmGraph *graph, gboolean inv)
 {
+	g_return_if_fail (graph != NULL);
+	g_return_if_fail (GPM_IS_GRAPH (graph));
 	graph->priv->invert_y = inv;
 }
 
 /**
  * gpm_graph_set_data:
- * @graph: This simple graph class instance
+ * @graph: This graph class instance
  * @list: The GList values to be plotted on the graph
  *
  * Sets the data for the graph. You MUST NOT free the list before the widget.
@@ -201,9 +268,26 @@ gpm_graph_set_invert_y (GpmGraph *graph, gboolean inv)
 void
 gpm_graph_set_data (GpmGraph *graph, GList *list)
 {
+	g_return_if_fail (graph != NULL);
+	g_return_if_fail (GPM_IS_GRAPH (graph));
 	graph->priv->list = list;
 }
 
+/**
+ * gpm_graph_set_events:
+ * @graph: This graph class instance
+ * @list: The GList events to be plotted on the graph
+ *
+ * Sets the data for the graph. You MUST NOT free the list before the widget.
+ **/
+void
+gpm_graph_set_events (GpmGraph *graph, GList *list)
+{
+	g_return_if_fail (graph != NULL);
+	g_return_if_fail (GPM_IS_GRAPH (graph));
+	graph->priv->events = list;
+}
+
 /**
  * gpm_get_axis_label:
  * @axis: The axis type, e.g. GPM_GRAPH_TYPE_TIME
@@ -252,7 +336,7 @@ gpm_get_axis_label (GpmGraphAxisType axis, int value)
 
 /**
  * gpm_graph_draw_grid:
- * @graph: This simple graph class instance
+ * @graph: This graph class instance
  * @cr: Cairo drawing context
  *
  * Draw the 10x10 dotted grid onto the graph.
@@ -292,7 +376,7 @@ gpm_graph_draw_grid (GpmGraph *graph, cairo_t *cr)
 
 /**
  * gpm_graph_draw_labels:
- * @graph: This simple graph class instance
+ * @graph: This graph class instance
  * @cr: Cairo drawing context
  *
  * Draw the X and the Y labels onto the graph.
@@ -305,8 +389,8 @@ gpm_graph_draw_labels (GpmGraph *graph, cairo_t *cr)
 	gint value;
 	float divwidth  = (float)graph->priv->box_width / 10.0f;
 	float divheight = (float)graph->priv->box_height / 10.0f;
-	gint length_x = graph->priv->stop_x;
-	gint length_y = graph->priv->stop_y;
+	int length_x = graph->priv->stop_x - graph->priv->start_x;
+	int length_y = graph->priv->stop_y - graph->priv->start_y;
 	cairo_text_extents_t extents;
 	float offsetx = 0;
 	float offsety = 0;
@@ -320,9 +404,9 @@ gpm_graph_draw_labels (GpmGraph *graph, cairo_t *cr)
 	for (a=0; a<11; a++) {
 		b = graph->priv->box_x + (a * divwidth);
 		if (graph->priv->invert_x) {
-			value = (length_x / 10) * (10 - a);
+			value = (length_x / 10) * (10 - a) + graph->priv->start_x;
 		} else {
-			value = (length_x / 10) * a;
+			value = ((length_x / 10) * a) + graph->priv->start_x;
 		}
 		text = gpm_get_axis_label (graph->priv->axis_x, value);
 
@@ -347,9 +431,9 @@ gpm_graph_draw_labels (GpmGraph *graph, cairo_t *cr)
 	for (a=0; a<11; a++) {
 		b = graph->priv->box_y + (a * divheight);
 		if (graph->priv->invert_y) {
-			value = (length_y / 10) * a;
+			value = (length_y / 10) * a - graph->priv->start_y;
 		} else {
-			value = (length_y / 10) * (10 - a);
+			value = (length_y / 10) * (10 - a) - graph->priv->start_y;
 		}
 		text = gpm_get_axis_label (graph->priv->axis_y, value);
 
@@ -373,43 +457,43 @@ gpm_graph_draw_labels (GpmGraph *graph, cairo_t *cr)
 
 /**
  * gpm_graph_check_range:
- * @graph: This simple graph class instance
+ * @graph: This graph class instance
  *
  * Checks all points are displayable on the graph, nobbling if required.
  **/
 static void
 gpm_graph_check_range (GpmGraph *graph)
 {
-	if (! graph->priv->list) {
+	if (graph->priv->list == NULL || graph->priv->list->data == NULL) {
 		gpm_debug ("no data");
 		return;
 	}
-	GpmDataPoint *new = NULL;
+	GpmInfoDataPoint *new = NULL;
 	GList *l;
 	for (l=graph->priv->list; l != NULL; l=l->next) {
-		new = (GpmDataPoint *) l->data;
-		if (new->x < 0) {
-			gpm_warning ("point out of range (x=%i)", new->x);
-			new->x = 0;
+		new = (GpmInfoDataPoint *) l->data;
+		if (new->time < graph->priv->start_x) {
+			gpm_warning ("point out of range (x=%i)", new->time);
+			new->time = graph->priv->start_x;
 		}
-		if (new->x > graph->priv->stop_x) {
-			gpm_warning ("point out of range (x=%i)", new->x);
-			new->x = graph->priv->stop_x;
+		if (new->time > graph->priv->stop_x) {
+			gpm_warning ("point out of range (x=%i)", new->time);
+			new->time = graph->priv->stop_x;
 		}
-		if (new->y < 0) {
-			gpm_warning ("point out of range (y=%i)", new->y);
-			new->y = 0;
+		if (new->value < graph->priv->start_y) {
+			gpm_warning ("point out of range (y=%i)", new->value);
+			new->value = graph->priv->start_y;
 		}
-		if (new->y > graph->priv->stop_y) {
-			gpm_warning ("point out of range (y=%i)", new->y);
-			new->y = graph->priv->stop_y;
+		if (new->value > graph->priv->stop_y) {
+			gpm_warning ("point out of range (y=%i)", new->value);
+			new->value = graph->priv->stop_y;
 		}
 	}
 }
 
 /**
  * gpm_graph_auto_range:
- * @graph: This simple graph class instance
+ * @graph: This graph class instance
  *
  * Autoranges the graph axis depending on the axis type, and the maximum
  * value of the data. We have to be careful to choose a number that gives good
@@ -418,57 +502,88 @@ gpm_graph_check_range (GpmGraph *graph)
 static void
 gpm_graph_auto_range (GpmGraph *graph)
 {
-	if (! graph->priv->list) {
+	if (graph->priv->list == NULL || graph->priv->list->data == NULL) {
 		gpm_debug ("no data");
+		graph->priv->start_x = 0;
+		graph->priv->start_y = 0;
+		graph->priv->stop_x = 10;
+		graph->priv->stop_y = 10;
 		return;
 	}
 
 	int biggest_x = 0;
 	int biggest_y = 0;
-	GpmDataPoint *new = NULL;
+	int smallest_x = 999999;
+	int smallest_y = 999999;
+	GpmInfoDataPoint *new = NULL;
 	GList *l;
 	for (l=graph->priv->list; l != NULL; l=l->next) {
-		new = (GpmDataPoint *) l->data;
-		if (new->x > biggest_x) {
-			biggest_x = new->x;
+		new = (GpmInfoDataPoint *) l->data;
+		if (new->time > biggest_x) {
+			biggest_x = new->time;
 		}
-		if (new->y > biggest_y) {
-			biggest_y = new->y;
+		if (new->value > biggest_y) {
+			biggest_y = new->value;
 		}
+		if (new->time < smallest_x) {
+			smallest_x = new->time;
+		}
+		if (new->value < smallest_y) {
+			smallest_y = new->value;
+		}
+	}
+
+	/* do we autorange the start (so it starts at non-zero)? */
+	if (graph->priv->autorange_x) {
+		/* x is always time and always autoranges to the minute scale */
+		smallest_x = (smallest_x / 60) * 60;
+		if (smallest_x < 60) {
+			smallest_x = 0;
+		}
+		graph->priv->start_x = smallest_x;
+	} else {
+		graph->priv->start_x = 0;
 	}
 
 	/* x */
 	if (graph->priv->axis_x == GPM_GRAPH_TYPE_PERCENTAGE) {
 		graph->priv->stop_x = 100;
 	} else if (graph->priv->axis_x == GPM_GRAPH_TYPE_TIME) {
-		graph->priv->stop_x = ((biggest_x/60)+1)*60;
-		if (graph->priv->stop_x > 60) {
-			graph->priv->stop_x = ((biggest_x/(10*60))+1)*(10*60);
-		}
+		graph->priv->stop_x = ((biggest_x / (10 * 60)) + 1) * (10 * 60) + smallest_x;
 	} else if (graph->priv->axis_x == GPM_GRAPH_TYPE_RATE) {
-		graph->priv->stop_x = ((biggest_x/10000)+2)*10000;
+		graph->priv->stop_x = ((biggest_x / 10000) + 2) * 10000 + smallest_x;
 		if (graph->priv->stop_x < 10000) {
-			graph->priv->stop_x = 10000;
+			graph->priv->stop_x = 10000 + smallest_x;
 		}
 	} else {
-		graph->priv->stop_x = ((biggest_x/10)+1)*10;
+		graph->priv->stop_x = ((biggest_x / 10) + 1) * 10 + smallest_x;
 	}
 
 	/* y */
 	if (graph->priv->axis_y == GPM_GRAPH_TYPE_PERCENTAGE) {
+		graph->priv->start_y = 0;
 		graph->priv->stop_y = 100;
 	} else if (graph->priv->axis_y == GPM_GRAPH_TYPE_TIME) {
-		graph->priv->stop_y = ((biggest_y/60)+2)*60;
+		graph->priv->start_y = 0;
+		graph->priv->stop_y = ((biggest_y / 60) + 2)* 60;
 		if (graph->priv->stop_y > 60) {
-			graph->priv->stop_y = ((biggest_y/(10*60))+2)*(10*60);
+			graph->priv->stop_y = ((biggest_y / (10 * 60)) + 2) * (10 * 60);
+		}
+		if (graph->priv->stop_y < 60) {
+			graph->priv->stop_y = 60;
 		}
 	} else if (graph->priv->axis_y == GPM_GRAPH_TYPE_RATE) {
-		graph->priv->stop_y = ((biggest_y/10000)+2)*10000;
+		graph->priv->start_y = 0;
+		graph->priv->stop_y = ((biggest_y / 10000) + 2) * 10000;
 		if (graph->priv->stop_y < 10000) {
 			graph->priv->stop_y = 10000;
 		}
 	} else {
-		graph->priv->stop_y = ((biggest_y/10)+1)*10;
+		graph->priv->start_y = 0;
+		graph->priv->stop_y = ((biggest_y / 10) + 1) * 10;
+		if (graph->priv->stop_y < 10) {
+			graph->priv->stop_y = 10;
+		}
 	}
 }
 
@@ -490,18 +605,30 @@ gpm_graph_set_colour (cairo_t *cr, GpmGraphColour colour)
 		cairo_set_source_rgb (cr, 1, 0, 0);
 	} else if (colour == GPM_GRAPH_COLOUR_BLUE) {
 		cairo_set_source_rgb (cr, 0, 0, 1);
-	} else if (colour == GPM_GRAPH_COLOUR_PURPLE) {
+	} else if (colour == GPM_GRAPH_COLOUR_GREEN) {
+		cairo_set_source_rgb (cr, 0, 1, 0);
+	} else if (colour == GPM_GRAPH_COLOUR_MAGENTA) {
 		cairo_set_source_rgb (cr, 1, 0, 1);
 	} else if (colour == GPM_GRAPH_COLOUR_YELLOW) {
 		cairo_set_source_rgb (cr, 1, 1, 0);
+	} else if (colour == GPM_GRAPH_COLOUR_CYAN) {
+		cairo_set_source_rgb (cr, 0, 1, 1);
+	} else if (colour == GPM_GRAPH_COLOUR_GREY) {
+		cairo_set_source_rgb (cr, 0.8, 0.8, 0.8);
 	} else if (colour == GPM_GRAPH_COLOUR_DARK_RED) {
 		cairo_set_source_rgb (cr, 0.5, 0, 0);
 	} else if (colour == GPM_GRAPH_COLOUR_DARK_BLUE) {
 		cairo_set_source_rgb (cr, 0, 0, 0.5);
-	} else if (colour == GPM_GRAPH_COLOUR_DARK_PURPLE) {
+	} else if (colour == GPM_GRAPH_COLOUR_DARK_GREEN) {
+		cairo_set_source_rgb (cr, 0, 0.5, 0);
+	} else if (colour == GPM_GRAPH_COLOUR_DARK_MAGENTA) {
 		cairo_set_source_rgb (cr, 0.5, 0, 0.5);
 	} else if (colour == GPM_GRAPH_COLOUR_DARK_YELLOW) {
 		cairo_set_source_rgb (cr, 0.5, 0.5, 0);
+	} else if (colour == GPM_GRAPH_COLOUR_DARK_CYAN) {
+		cairo_set_source_rgb (cr, 0, 0.5, 0.5);
+	} else if (colour == GPM_GRAPH_COLOUR_DARK_GREY) {
+		cairo_set_source_rgb (cr, 0.5, 0.5, 0.5);
 	} else {
 		gpm_critical_error ("Unknown colour!");
 	}
@@ -530,7 +657,7 @@ gpm_graph_draw_dot (cairo_t *cr, float x, float y, GpmGraphColour colour)
 
 /**
  * gpm_graph_get_pos_on_graph:
- * @graph: This simple graph class instance
+ * @graph: This graph class instance
  * @data_x: The data X-coordinate
  * @data_y: The data Y-coordinate
  * @x: The returned X position on the cairo surface
@@ -539,79 +666,134 @@ gpm_graph_draw_dot (cairo_t *cr, float x, float y, GpmGraphColour colour)
 static void
 gpm_graph_get_pos_on_graph (GpmGraph *graph, float data_x, float data_y, float *x, float *y)
 {
-	*x = graph->priv->box_x + (graph->priv->unit_x * data_x) + 1;
-	*y = graph->priv->box_y + (graph->priv->unit_y * (float)(graph->priv->stop_y - data_y)) + 1.5;
+	*x = graph->priv->box_x + (graph->priv->unit_x * (data_x - graph->priv->start_x)) + 1;
+	*y = graph->priv->box_y + (graph->priv->unit_y * (float)(graph->priv->stop_y - (data_y - graph->priv->start_y))) + 1.5;
+}
+
+/**
+ * gpm_graph_interpolate_value:
+ * @this: The first data point
+ * @last: The other data point
+ * @xintersect: The x value (i.e. the x we provide)
+ * Return value: The interpolated value, or 0 if invalid
+ *
+ * Interpolates onto the graph in the y direction. If only supplied one point
+ * then don't interpolate.
+ **/
+static int
+gpm_graph_interpolate_value (GpmInfoDataPoint *this,
+			     GpmInfoDataPoint *last,
+			     int xintersect)
+{
+	int dy, dx;
+	/* we have no points */
+	if (! this) {
+		return 0;
+	}
+	/* we only have one point, don't interpolate */
+	if (! last) {
+		return this->value;
+	}
+	/* gradient */
+	dx = this->time - last->time;
+	dy = this->value - last->value;
+	float m = (float) dy / (float) dx;
+	/* y-intersect */
+	int c = (-m * (float) this->time) + this->value;
+	/* y = mx + c */
+	return (m * (float) xintersect) + c;
 }
 
 /**
  * gpm_graph_draw_line:
- * @graph: This simple graph class instance
+ * @graph: This graph class instance
  * @cr: Cairo drawing context
  *
  * Draw the data line onto the graph with a big green line. We should already
- * limit the data to < 100 values, so this shouldn't take too long.
+ * limit the data to < ~100 values, so this shouldn't take too long.
  **/
 static void
 gpm_graph_draw_line (GpmGraph *graph, cairo_t *cr)
 {
 	float oldx, oldy;
 	float newx, newy;
+	GpmInfoDataPoint *eventdata;
+	GList *l;
 
-	int length_x = graph->priv->stop_x;
-	int length_y = graph->priv->stop_y;
-
-	GList *link;
-	GpmDataPoint *old;
-
-	/* -3 is so we can keep the lines inside the box at both extremes */
-	graph->priv->unit_x = (float)(graph->priv->box_width - 3) / (float) length_x;
-	graph->priv->unit_y = (float)(graph->priv->box_height - 3) / (float) length_y;
-
-	if (! graph->priv->list) {
+	if (graph->priv->list == NULL || graph->priv->list->data == NULL) {
 		gpm_debug ("no data");
 		return;
 	}
 
-	gpm_graph_check_range (graph);
+	/* I don't think we need to do this anymore.... */
+	if (FALSE) {
+		gpm_graph_check_range (graph);
+	}
 
 	cairo_save (cr);
 
-	/* do the line */
-	link = graph->priv->list;
-	old = link->data;
-	gpm_graph_get_pos_on_graph (graph, old->x, old->y, &oldx, &oldy);
-	while (link->next && link->next->data) {
-		GpmDataPoint *new = link->next->data;
-		if (! new->info_point) {
-			/* do line */
-			gpm_graph_get_pos_on_graph (graph, new->x, new->y, &newx, &newy);
-			cairo_move_to (cr, oldx, oldy);
-			cairo_line_to (cr, newx, newy);
-			cairo_set_line_width (cr, 2);
-			gpm_graph_set_colour (cr, new->colour);
-			cairo_stroke (cr);
-			/* save old */
-			oldx = newx;
-			oldy = newy;
-		}
-		link=link->next;
+	/* do the line on the graph */
+	GpmInfoDataPoint *new;
+	l=graph->priv->list;
+	new = (GpmInfoDataPoint *) l->data;
+	gpm_graph_get_pos_on_graph (graph, new->time, new->value, &oldx, &oldy);
+	for (l=l->next; l != NULL; l=l->next) {
+		new = (GpmInfoDataPoint *) l->data;
+		/* do line */
+		gpm_graph_get_pos_on_graph (graph, new->time, new->value, &newx, &newy);
+		cairo_move_to (cr, oldx, oldy);
+		cairo_line_to (cr, newx, newy);
+		cairo_set_line_width (cr, 2);
+		gpm_graph_set_colour (cr, new->colour);
+		cairo_stroke (cr);
+		/* save old */
+		oldx = newx;
+		oldy = newy;
 	}
 
-	/* do the events on the graph */
-	link = graph->priv->list;
-	old = link->data;
-	gpm_graph_get_pos_on_graph (graph, old->x, old->y, &oldx, &oldy);
-	while (link->next && link->next->data) {
-		GpmDataPoint *new = link->next->data;
-		if (new->info_point) {
-			gpm_graph_draw_dot (cr, oldx, oldy, new->colour);
-		} else {
-			/* save old */
-			gpm_graph_get_pos_on_graph (graph, new->x, new->y, &newx, &newy);
-			oldx = newx;
-			oldy = newy;
+	/* do the events on the graph if we enabled the legend */
+	if (graph->priv->use_legend) {
+		int previous_point = 0;
+		int prevpos = -1;
+		GpmInfoDataPoint *point_this = NULL;
+		GpmInfoDataPoint *point_last = NULL;
+
+		/* we track the list so we can put the point on the line */
+		GList *l2 = graph->priv->list;
+		if (l2) {
+			point_this = (GpmInfoDataPoint *) l2->data;
+			l2 = l2->next;
+		}
+		for (l=graph->priv->events; l != NULL; l=l->next) {
+			eventdata = (GpmInfoDataPoint *) l->data;
+			/* If we have valid list data, go through the list data
+			   until we get a data point time value greater than the
+			   event we have. */
+			while (l2 && eventdata->time > point_this->time) {
+				point_last = point_this;
+				point_this = (GpmInfoDataPoint *) l2->data;
+				l2 = l2->next;
+			}
+			/* Interpolate in the y direction with the two
+			   previous points. This should be 100% accurate */
+			int pos = gpm_graph_interpolate_value (point_this,
+							       point_last,
+							       eventdata->time);
+			gpm_graph_get_pos_on_graph (graph, eventdata->time,
+						    pos, &newx, &newy);
+			/* don't overlay the points, stack vertically */
+			if (abs (prevpos - newx) < 8) {
+				previous_point++;
+			} else {
+				previous_point = 0;
+			}
+			newy -= (8 * previous_point);
+			/* only do the event dot, if it's going to fit on the graph */
+			if (eventdata->time > graph->priv->start_x) {
+				gpm_graph_draw_dot (cr, newx, newy, eventdata->colour);
+			}
+			prevpos = newx;
 		}
-		link=link->next;
 	}
 
 	cairo_restore (cr);
@@ -658,7 +840,8 @@ gpm_graph_draw_legend (cairo_t *cr, int x, int y, int width, int height)
 	gpm_graph_draw_bounding_box (cr, x, y, width, height);
 	y_count = y + 10;
 	for (a=0; a<GPM_GRAPH_EVENT_LAST; a++) {
-		desc = 	gpm_graph_event_description (a, &colour);
+		desc = gpm_graph_event_description (a);
+		colour = gpm_graph_event_colour (a);
 		gpm_graph_draw_dot (cr, x + 8, y_count, colour);
 		cairo_move_to (cr, x + 8 + 10, y_count + 3);
 		cairo_show_text (cr, desc);
@@ -666,9 +849,45 @@ gpm_graph_draw_legend (cairo_t *cr, int x, int y, int width, int height)
 	}
 }
 
+/**
+ * gpm_graph_legend_calculate_width:
+ * @graph: This graph class instance
+ * @cr: Cairo drawing context
+ * Return value: The width of the legend, including borders.
+ *
+ * We have to find the maximum size of the text so we know the width of the
+ * legend box. We can't hardcode this as the dpi or font size might differ
+ * from machine to machine.
+ **/
+static float
+gpm_graph_legend_calculate_width (GpmGraph *graph, cairo_t *cr)
+{
+	g_return_val_if_fail (graph != NULL, 0.0f);
+	g_return_val_if_fail (GPM_IS_GRAPH (graph), 0.0f);
+
+	int a;
+	const char *desc;
+	cairo_text_extents_t extents;
+	float max_width = 0.0f;
+
+	cairo_set_font_options (cr, graph->priv->options);
+	for (a=0; a<GPM_GRAPH_EVENT_LAST; a++) {
+		desc = gpm_graph_event_description (a);
+		cairo_text_extents (cr, desc, &extents);
+		if (max_width < extents.width) {
+			max_width = extents.width;
+		}
+	}
+
+	/* add for borders */
+	max_width += 25;
+
+	return max_width;
+}
+
 /**
  * gpm_graph_draw_graph:
- * @graph: This simple graph class instance
+ * @graph: This graph class instance
  * @cr: Cairo drawing context
  *
  * Draw the complete graph, with the box, the grid, the labels and the line.
@@ -678,10 +897,15 @@ gpm_graph_draw_graph (GtkWidget *graph_widget, cairo_t *cr)
 {
 	int legend_x = 0;
 	int legend_y = 0;
-	int legend_height = GPM_GRAPH_EVENT_LAST * 20;
-	int legend_width = 77;
+	int legend_height;
+	int legend_width;
 
 	GpmGraph *graph = (GpmGraph*) graph_widget;
+	g_return_if_fail (graph != NULL);
+	g_return_if_fail (GPM_IS_GRAPH (graph));
+
+	legend_width = gpm_graph_legend_calculate_width (graph, cr);
+	legend_height = GPM_GRAPH_EVENT_LAST * 20;
 
 	cairo_save (cr);
 
@@ -689,11 +913,13 @@ gpm_graph_draw_graph (GtkWidget *graph_widget, cairo_t *cr)
 	graph->priv->box_y = 5;
 
 	if (graph->priv->use_legend) {
-		graph->priv->box_width = graph_widget->allocation.width - (3 + legend_width + 5 + graph->priv->box_x);
+		graph->priv->box_width = graph_widget->allocation.width -
+					 (3 + legend_width + 5 + graph->priv->box_x);
 		legend_x = graph->priv->box_x + graph->priv->box_width + 6;
 		legend_y = graph->priv->box_y;
 	} else {
-		graph->priv->box_width = graph_widget->allocation.width - (3 + graph->priv->box_x);
+		graph->priv->box_width = graph_widget->allocation.width -
+					 (3 + graph->priv->box_x);
 	}
 	graph->priv->box_height = graph_widget->allocation.height - (20 + graph->priv->box_y);
 
@@ -707,6 +933,12 @@ gpm_graph_draw_graph (GtkWidget *graph_widget, cairo_t *cr)
 
 	gpm_graph_auto_range (graph);
 
+	/* -3 is so we can keep the lines inside the box at both extremes */
+	int data_x = graph->priv->stop_x - graph->priv->start_x;
+	int data_y = graph->priv->stop_y - graph->priv->start_y;
+	graph->priv->unit_x = (float)(graph->priv->box_width - 3) / (float) data_x;
+	graph->priv->unit_y = (float)(graph->priv->box_height - 3) / (float) data_y;
+
 	gpm_graph_draw_labels (graph, cr);
 	gpm_graph_draw_line (graph, cr);
 
@@ -719,7 +951,7 @@ gpm_graph_draw_graph (GtkWidget *graph_widget, cairo_t *cr)
 
 /**
  * gpm_graph_expose:
- * @graph: This simple graph class instance
+ * @graph: This graph class instance
  * @event: The expose event
  *
  * Just repaint the entire graph widget on expose.
