@@ -37,30 +37,36 @@ G_BEGIN_DECLS
 
 G_BEGIN_DECLS
 
-gboolean gpm_hal_get_dbus_connection (DBusGConnection **connection);
+gboolean	 gpm_hal_get_dbus_connection		(DBusGConnection **connection);
 
-gboolean gpm_hal_has_power_management (void);
+gboolean	 gpm_hal_has_power_management		(void);
 
-gboolean gpm_hal_is_running (void);
-gboolean gpm_hal_is_laptop (void);
-gboolean gpm_hal_is_on_ac (void);
+gboolean	 gpm_hal_is_running			(void);
+gboolean	 gpm_hal_is_laptop			(void);
+gboolean	 gpm_hal_is_on_ac			(void);
+gboolean	 gpm_hal_can_suspend			(void);
+gboolean	 gpm_hal_suspend			(gint		 wakeup);
+gboolean	 gpm_hal_can_hibernate			(void);
+gboolean	 gpm_hal_hibernate			(void);
+gboolean	 gpm_hal_shutdown			(void);
+gboolean	 gpm_hal_enable_power_save		(gboolean	 enable);
+gboolean	 gpm_hal_device_get_bool		(const gchar	*udi,
+							 const gchar	*key,
+							 gboolean	*value);
+gboolean	 gpm_hal_device_get_string		(const gchar	*udi,
+							 const gchar	*key,
+							 gchar		**value);
+gboolean	 gpm_hal_device_get_int			(const gchar	*udi,
+							 const gchar	*key,
+							 gint		*value);
+gboolean	 gpm_hal_find_device_capability		(const gchar	*capability,
+							 gchar		***value);
 
-gboolean gpm_hal_can_suspend (void);
-gboolean gpm_hal_suspend (gint wakeup);
-gboolean gpm_hal_can_hibernate (void);
-gboolean gpm_hal_hibernate (void);
-gboolean gpm_hal_shutdown (void);
-
-gboolean gpm_hal_enable_power_save (gboolean enable);
-
-gboolean gpm_hal_device_get_bool (const gchar *udi, const gchar *key, gboolean *value);
-gboolean gpm_hal_device_get_string (const gchar *udi, const gchar *key, gchar **value);
-gboolean gpm_hal_device_get_int (const gchar *udi, const gchar *key, gint *value);
-gboolean gpm_hal_find_device_capability (const gchar *capability, gchar ***value);
-
-gint gpm_hal_num_devices_of_capability (const gchar *capability);
-gint gpm_hal_num_devices_of_capability_with_value (const gchar *capability, const gchar *key, const gchar *value);
-void gpm_hal_free_capability (gchar **value);
+gint		 gpm_hal_num_devices_of_capability	(const gchar	*capability);
+gint		 gpm_hal_num_devices_of_capability_with_value (const gchar *capability,
+							 const gchar	*key,
+							 const gchar	*value);
+void		 gpm_hal_free_capability		(gchar		**value);
 
 G_END_DECLS
 
