@@ -48,9 +48,10 @@ typedef struct
 } GpmInfoDataClass;
 
 typedef struct {
-	int		time;	/* seconds */
-	int		value;
-	int		colour;
+	int		 time;	/* seconds */
+	int		 value;
+	int		 colour;
+	char		*desc; /* description, or NULL if missing */
 } GpmInfoDataPoint;
 
 GType			 gpm_info_data_get_type		(void);
@@ -64,7 +65,8 @@ void			 gpm_info_data_add		(GpmInfoData	*info_data,
 void			 gpm_info_data_add_always	(GpmInfoData	*info_data,
 							 int		 time,
 							 int		 value,
-							 int		 colour);
+							 int		 colour,
+							 const char	*desc);
 void			 gpm_info_data_limit_time	(GpmInfoData	*info_data,
 							 int		 max_num);
 void			 gpm_info_data_limit_dilute	(GpmInfoData	*info_data,
