@@ -56,7 +56,7 @@ static void gpm_exit (GpmManager *manager);
  **/
 static gboolean
 gpm_object_register (DBusGConnection *connection,
-		     GObject         *object)
+		     GObject	     *object)
 {
 	DBusGProxy *bus_proxy = NULL;
 	GError *error = NULL;
@@ -114,14 +114,13 @@ main (int argc, char *argv[])
 	GnomeClientFlags flags;
 	DBusGConnection *system_connection;
 	DBusGConnection *session_connection;
-	gboolean         verbose = FALSE;
-	gboolean         no_daemon = FALSE;
+	gboolean	 verbose = FALSE;
+	gboolean	 no_daemon = FALSE;
 	GpmManager      *manager = NULL;
-	GError          *error = NULL;
+	GError		*error = NULL;
 	GOptionContext  *context;
  	GnomeProgram    *program;
 
-
 	const GOptionEntry options[] = {
 		{ "no-daemon", '\0', 0, G_OPTION_ARG_NONE, &no_daemon,
 		  N_("Do not daemonize"), NULL },
@@ -160,7 +159,6 @@ main (int argc, char *argv[])
 		g_thread_init (NULL);
 	dbus_g_thread_init ();
 
-
 	gpm_debug_init (verbose);
 
 	/* check dbus connections, exit if not valid */
@@ -169,9 +167,7 @@ main (int argc, char *argv[])
 		gpm_warning ("%s", error->message);
 		g_error_free (error);
 		gpm_critical_error ("This program cannot start until you start the dbus "
-				    "<i>system</i> service.\n\n"
-				    "This is usually started by your operating system by "
-				    "default, and is normally called messagebus.\n\n"
+				    "<i>system</i> service.\n"
 				    "It is <b>strongly recommended</b> you reboot your compter "
 				    "after starting messagebus.");
 		/* abort at this point */
@@ -185,10 +181,7 @@ main (int argc, char *argv[])
 		gpm_critical_error ("This program cannot start until you start the "
 				    "dbus <i>session</i> service.\n\n"
 				    "This is usually started automatically in X "
-				    "or gnome startup when you start a new session.\n\n"
-				    "You can launch the session dbus-daemon manually "
-				    "with this command:\n\n"
-				    "<b>eval `dbus-launch --auto-syntax`</b>");
+				    "or gnome startup when you start a new session.");
 		/* abort at this point */
 		exit (1);
 	}
